/* This file is part of the KDE project
   Copyright (C) 1999 David Faure <faure@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef dirlister_h
#define dirlister_h

#include <kfileitem.h>
#include <kdirnotify.h>

#include <qtimer.h>
#include <qstringlist.h>
#include <qlist.h>
#include <qregexp.h>

#include <kurl.h>
#include <kio/global.h>

namespace KIO { class Job; class ListJob; }

class DirLister : public QObject, public KDirNotify
{
  Q_OBJECT
public:
  DirLister( bool _delayedMimeTypes = false );
  virtual ~DirLister();

  virtual void openURL( const KURL& _url, bool _showDotFiles, bool _keep = false );

  virtual void stop();

  virtual const KURL & url() const { return m_url; }

  virtual bool setURL( const KURL& url );

  virtual void updateDirectory( const KURL& dir );

  void listDirectory();

  void setAutoUpdate( bool enable );

  bool autoUpdate() const;

  virtual void setShowingDotFiles( bool _showDotFiles );

  virtual bool showingDotFiles() const;

  KFileItem* find( const KURL& _url ) const;

  KFileItem* findByName( const QString& name ) const;

  QList<KFileItem> & items() { return m_lstFileItems; }

  KFileItem * rootItem() const { return m_rootFileItem; }

  KIO::ListJob * job() const { return m_job; }

  void setDirOnlyMode( bool dirsOnly ) { m_bDirOnlyMode = dirsOnly; }

  bool dirOnlyMode() const { return m_bDirOnlyMode; }

  void setNameFilter(const QString&);

  bool matchesFilter( const QString& name ) const;

  virtual void FilesAdded( const KURL & directory );

  virtual void FilesRemoved( const KURL::List & fileList );

  virtual void FilesChanged( const KURL::List & fileList );

  bool isFinished() const { return (m_job == 0); }

signals:
  void started( const QString& _url );

  void completed();
  void canceled();

  void redirection( const KURL & url );

  void clear();
  void newItems( const KFileItemList & items );
  void deleteItem( KFileItem * _fileItem );
  void refreshItems( const KFileItemList & items );
  void closeView();

protected slots:
  void slotResult( KIO::Job * );
  void slotEntries( KIO::Job*, const KIO::UDSEntryList& );
  void slotUpdateResult( KIO::Job * );
  void slotUpdateEntries( KIO::Job*, const KIO::UDSEntryList& );
  void slotRedirection( KIO::Job *, const KURL & url );

  void slotDirectoryDirty( const QString& _dir );
  void slotFileDirty( const QString& _file );

  void slotURLDirty( const KURL& dir );

protected:

  virtual KFileItem * createFileItem( const KIO::UDSEntry&, const KURL&url,
                                      bool determineMimeTypeOnDemand );

  virtual bool matchesFilter( const KFileItem * ) const;

  void forgetDirs();

  void deleteUnmarkedItems();

  bool validURL( const KURL& ) const;

  KURL m_url;

  KIO::ListJob * m_job;

  QList<KFileItem> m_lstFileItems;
  KFileItem * m_rootFileItem;

  KURL::List m_lstDirs;

  bool m_isShowingDotFiles;
  bool m_bComplete;

  QValueList<KIO::UDSEntry> m_buffer;

  bool m_bDirOnlyMode;

  bool m_bDelayedMimeTypes;

  QList<QRegExp> m_lstFilters;

  class DirListerPrivate;
  DirListerPrivate * d;
};

#endif

