/***************************************************************************
                          diroperator.h  -  description
                             -------------------
    begin                : Sun Feb 4 2001
    copyright            : (C) 2001 by Rene Mrten & Christian Fricke
    email                : delta_x@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// -*- c++ -*-
/* This file is part of the KDE libraries
    Copyright (C) 1999 Stephan Kulow <coolo@kde.org>
		  2000 Carsten Pfeiffer <pfeiffer@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#ifndef DIROPERATOR_H_
#define DIROPERATOR_H_

#include <qobject.h>
#include <qstack.h>

#include <kaction.h>
#include <kcompletion.h>

#include <kfileview.h>
#include <kfileviewitem.h>
#include <kfileitem.h>
#include <kfile.h>

#include <qpopupmenu.h>
#include <qtimer.h>

#include <kaction.h>
#include "filereader.h"
#include <qwidgetstack.h>
#include <kprogress.h>

class DirOperator : public QWidget {
    Q_OBJECT
	
 public:
    DirOperator(const KURL& urlName = KURL(),
		 QWidget *parent = 0, const char* name = 0);
    virtual ~DirOperator();

	  KFileItem* findByName( const QString& name ) const;
	  void  keyPressEvent( QKeyEvent *);	

    void setShowHiddenFiles ( bool s ) { showHiddenAction->setChecked( s ); }
    bool showHiddenFiles () const { return showHiddenAction->isChecked(); }

		void setDirsExtra();
		
    void close();

    void setNameFilter(const QString& filter);

    KURL url() const;

    void setURL(const KURL& url, bool clearforward);

    void setCurrentItem( const QString& filename );

    void setView(KFileView *view,bool mod = true);
    const KFileView * view() const { return fileView; }
    void setView(KFile::FileView view,bool mod=true);

    void setSorting( QDir::SortSpec );
    QDir::SortSpec sorting() const { return mySorting; }

    bool isRoot() const;

    FileReader *fileReader() const { return dir; }

    void setMode( KFile::Mode m );
    KFile::Mode mode() const;

    void setPreviewWidget(const QWidget *w);

    const KFileViewItemList * selectedItems()
    	const {	
    					return ( fileView ? fileView->selectedItems() : 0L );
    }
    inline bool isSelected( const KFileViewItem *item ) const {
	return ( fileView ? fileView->isSelected( item ) : false );
    }


    int numDirs() const;
    int numFiles() const;

    KCompletion * completionObject() const {	return const_cast<KCompletion *>( &myCompletion );
    }

    KCompletion *dirCompletionObject() const {	return const_cast<KCompletion *>( &myDirCompletion );
    }
    KActionCollection * actionCollection() const { return myActionCollection; }

    virtual void readConfig( KConfig *, const QString& group = QString::null );

    virtual void saveConfig( KConfig *, const QString& group = QString::null );

    void setOnlyDoubleClickSelectsFiles( bool enable );

    bool onlyDoubleClickSelectsFiles() const;

// Arno///////////////////////
		void setSelection(QString);
		void setUnSelection(QString);
		void setSelectionAll();
		void setSelectionInvert();
		void setSelectionNone();
/////////////////////////////

protected:
    void setFileReader( FileReader *reader );
    void resizeEvent( QResizeEvent * );
    void setupActions();
    void updateSortActions();
    void updateViewActions();
    void setupMenu();
    void prepareCompletionObjects();
    void insertIntoView(const KFileItemList& items);

 private:
 		QString tmpFile;

    QStack<KURL> backStack;

    QStack<KURL> forwardStack;

    static KURL *lastDirectory;

    FileReader *dir;

    KCompletion myCompletion;
    KCompletion myDirCompletion;
    bool myCompleteListDirty;
    QDir::SortSpec mySorting;

    void checkPath(const QString& txt, bool takeFiles = false);

    void connectView(KFileView *);

    bool finished;

    KFileView *fileView;
    KFileView *oldView;

    KFileViewItemList pendingMimeTypes;

    int viewKind;
    int defaultView;

    KFile::Mode myMode;
    KProgress *progress;

    QWidget *myPreview;    // temporary pointer for the preview widget

    // actions for the popupmenus
    KActionMenu *actionMenu;

    KAction 	*backAction;
    KAction 	*forwardAction;
    KAction 	*homeAction;
    KAction 	*upAction;
    KAction 	*reloadAction;
    KActionSeparator *actionSeparator;
    KAction 	*mkdirAction;
    KAction 	*renFileAction;
    KAction 	*openWithAction;
    KAction 	*bookmarkAction;
    KAction 	*setFilterAction;
    KAction 	*emailAction;

    KActionMenu *sortActionMenu;
    KRadioAction *byNameAction;
    KRadioAction *byDateAction;
    KRadioAction *bySizeAction;
    KToggleAction *reverseAction;
    KToggleAction *dirsFirstAction;
    KToggleAction *caseInsensitiveAction;

    KActionMenu *viewActionMenu;
    KRadioAction *shortAction;
    KRadioAction *detailedAction;
    KToggleAction *showHiddenAction;
    KToggleAction *separateDirsAction;

    KActionCollection *myActionCollection;

 public slots:
    void back();
    void forward();
    void home();
    void cdUp();
    void rereadDir();
    void mkdir();
    void rename();
//Arno/////////////////////
    void openWith();
    void bookMark();
		void setFilter();
		void sendMail();
//////////////////////////
    QString makeCompletion(const QString&);
    QString makeDirCompletion(const QString&);
    void slotToggleHidden( bool );
    void dropps(QStringList);
    void highlightFile(const KFileViewItem* i) { emit fileHighlighted( i );}//tmpFile.sprintf(i->urlString()); }

  protected slots:
    void resetCursor();
    void readNextMimeType();
    void pathChanged();
    void filterChanged();
    void insertNewFiles(const KFileItemList &newone);
    void itemDeleted(KFileItem *);

    void selectDir(const KFileViewItem*);
    void selectFile(const KFileViewItem*);
    void activatedMenu( const KFileViewItem * );

    void sortByName() 		{ byNameAction->setChecked( true ); }
    void sortBySize() 		{ bySizeAction->setChecked( true ); }
    void sortByDate() 		{ byDateAction->setChecked( true ); }
    void sortReversed() 	{ reverseAction->setChecked( !reverseAction->isChecked() ); }
    void toggleDirsFirst() 	{ dirsFirstAction->setChecked( !dirsFirstAction->isChecked() ); }
    void toggleIgnoreCase() 	{ caseInsensitiveAction->setChecked( !caseInsensitiveAction->isChecked() ); }

    void deleteOldView();

    void slotCompletionMatch(const QString&);
protected:
    virtual bool eventFilter( QObject *, QEvent * );

private slots:
    void slotDetailedView();
    void slotSimpleView();
//    void slotToggleHidden( bool );
    void slotToggleMixDirsAndFiles();
    void slotSortByName();
    void slotSortBySize();
    void slotSortByDate();
    void slotSortReversed();
    void slotToggleDirsFirst();
    void slotToggleIgnoreCase();

    void slotIOFinished();
		void mouseReleaseEvent(QMouseEvent *);

signals:
    void urlEntered(const KURL& );
    void updateInformation(int files, int dirs);
    void completion(const QString&);
    void finishedLoading();

    void fileHighlighted(const KFileViewItem*);
    void dirActivated(const KFileViewItem*);
    void fileSelected(const KFileViewItem*);

private:
    class DirOperatorPrivate;
    DirOperatorPrivate *d;
 signals:	
	void renameFile();
	void sendFile();
	void openWithSignal();
	void keyPress(QKeyEvent *);
	void hatFocus();
	void hatKeinFocus();
	void droppedFiles(QStringList);
};

#endif
