/***************************************************************************
                          filebasisview.h  -  description
                             -------------------
    begin                : Sun Feb 4 2001
    copyright            : (C) 2001 by Rene Mrten & Christian Fricke
    email                : delta_x@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __FILEBASISVIEW_H__
#define __FILEBASISVIEW_H__

#include <qcombobox.h>
#include <qlineedit.h>
#include <qstring.h>
#include <qobjectlist.h>
#include <qlayout.h>
#include <qtoolbutton.h>

#include <kdialogbase.h>
#include <kcolorbtn.h>

#include <kfile.h>
#include <kurl.h>
#include <kurlcombobox.h>
#include <kmimetype.h>
#include <kfileview.h>
#include <krun.h>
#include <qlabel.h>

#include "diroperator.h"
#include "config-kfile.h"
#include <kio/renamedlg.h>

#include <addressbook.h>


struct FileBasisViewPrivate;

class FileBasisView : public QDialog, public DCOPObject
{
    Q_OBJECT
    K_DCOP

public:
    FileBasisView(bool mod,const QString& urlName, const QString& filter,
		QWidget *parent=0, const char *name=0,
		bool modal = false);
		
		bool focused;

    ~FileBasisView();

	  void  keyPressEvent( QKeyEvent *);	
	  void  focusInEvent( QFocusEvent *);	

    void cdUp();
	  	
//		virtual QCStringList interfaces(void);
//		virtual QCStringList testtest(void);
	
	
		KFileItem* findByName( const QString& name ) const;
	
    KURL selectedURL() const;

    KURL::List selectedURLs() const;

    KURL baseURL() const;

    QString selectedFile() const;

    KFileViewItemList* selectedFiles() const;

    void setURL(const KURL &url, bool clearforward = true);

//    void setSelection(const QString& name);

    void setFilter(const QString& filter);

    QString currentFilter() const;

    void setFilterMimeType(const QString &label, const KMimeType::List &types, const KMimeType::Ptr &defaultType);

    KMimeType::Ptr currentFilterMimeType();

    void setPreviewWidget(const QWidget *w);

    virtual void show();

    void setMode( KFile::FileView m );
    void setToggleHidden(bool);
    QString currentDir();
    KURL url;
    void mkDir();
    KURL lastUrl;
    KFileViewItem currentItem();
    KFile::Mode mode() const;
		void setSelection(QString);
		void setSelectionAll();
		void setSelectionInvert();
		void setSelectionNone();

		bool checkIfTarBall(const KFileViewItem *);
//		void dragEnterEvent(QDragEnterEvent*);

    // NEW
    void changeFocusState(bool flag);
		QWidget* widget();
// der war vorher unten protected !!!!!!
    QPushButton *focus;
//    KColorButton *focus;
///////////////////////////////////////


signals:
//    void fileSelected(const QString&);
    void fileHighlighted(const QString&);
//    void selectionChanged();
//    void historyUpdate(bool, bool);
//    void filterChanged( const QString& filter );
		void focussed();
    void urlNew();
		void droppedFiles(QStringList);
    void changeStatus(const QString &);
								
protected:
    static KURL *lastDirectory;
		void highlightFile(const KFileViewItem*);
//
    QStringList history;
//
//    void addDirEntry(KFileViewItem *entry, bool disableUpdating);
//
//    void checkPath(const QString& txt, bool takeFiles = false);
//
//    void multiSelectionChanged();
//
    virtual void readConfig( KConfig *, const QString& group = QString::null );
    virtual void saveConfig( KConfig *, const QString& group = QString::null );
//
    virtual void readRecentFiles( KConfig * );

    virtual void saveRecentFiles( KConfig * );
//
//    KURL::List tokenize(const QString& line) const;

protected slots:
		void dragEnterEvent(QDragEnterEvent*);
		void dragMoveEvent(QDragMoveEvent*);
    void dropped(QStringList);
    void urlEntered( const KURL& );
//    void pathComboActivated( const KURL& url );
//    void pathComboReturnPressed( const QString& url );
//    void locationActivated( const QString& url );
//    void toolbarCallback(int);
//    void toolbarPressedCallback(int);
//    void slotFilterChanged();
//    void pathComboChanged( const QString& );
    void fileHighlighted(const KFileViewItem *i);

public slots:
    void fileSelected(const KFileViewItem *i);

protected slots:
//    void slotStatResult(KIO::Job* job);
    void slotLoadingFinished();

//    void dirCompletion( const QString& );
//    void fileCompletion( const QString& );
//
//    virtual void updateStatusLine(int dirs, int files);
//    virtual void slotOk();
    void slotChangeStatus(const QString &);

public slots:
    void setViewFocus();
    void setLostFocus();
    void driveActivated(const QString &);
		void slotRenameFile();
		void slotSendFile();
		void slotOpenWith();
		void bookMark();
		void slotShowBooks();		
		void bookMenuActivated (int);		
private:

    // cleanup the static variables
//    static void cleanup();
//    FileBasisView(const FileBasisView&);
//    FileBasisView operator=(const FileBasisView&);
		KRun *run;

protected:
    FileBasisViewPrivate *d;
    DirOperator *ops;
    DirOperator *ops1;
    DirOperator *ops2;
    bool autoDirectoryFollowing;
		QGridLayout	*upperBox;
		QGridLayout	*lowerBox;
		QVBoxLayout	*boxLayout;
		QComboBox		*driveList;
    QToolButton *bookButton;
		QString 		appName;
		QLabel 			*listInfoLabel;
		QWidget *widg;
//
    KURL::List& parseSelectedURLs() const;
    QString filenames;
    QString selection;
    QString completionHack;
    bool showStatusLine;
    QWidget *mainWidget;

    QList<KIO::StatJob> statJobs;

    bool completionLock;

    KURL::List urlList; //the list of selected urls

    KMimeType::Ptr defaultType; // the default mimetype to save as
    KMimeType::List mimetypes; //the list of possible mimetypes to save as

	  void  resizeEvent(QResizeEvent *);
    void updateInfoLabel();
};


class FileComboBox : public KURLComboBox
{
  Q_OBJECT

public:
  FileComboBox( bool rw, QWidget *parent=0, const char *name=0 )
      : KURLComboBox ( KURLComboBox::Files, rw, parent, name ) {}

  void setCompletion( const QString& );

};

#endif
