/***************************************************************************
                          kcommander2view.cpp  -  description
                             -------------------
    begin                : Sun Sep 17 12:25:22 GMT 2000
    copyright            : (C) 2000 by Rene Mrten / Christian Fricke
    email                : cfricke@codewizards.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qprinter.h>
#include <qpainter.h>
#include <qstyle.h>
#include <qurl.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmotifstyle.h>

// application specific includes
#include "kcommander2view.h"
#include "mainslots.h"
#include "kcommander2doc.h"
#include "kcommander2.h"
#include "inputbox.h"
#include <kapp.h>
#include <ktmainwindow.h>
#include <kmessagebox.h>
#include <kaccel.h>
#include <kseparator.h>
#include <kprocess.h>
#include <kfileviewitem.h>

#include <kio/job.h>

KCommander2View::KCommander2View(QWidget *parent, const char *name) : QWidget(parent, name)
{
  statusView = 0L;  // reset pointer to StatusWindow
	konsoleFlag = false;
  mainLayout = new QVBoxLayout(this,4);
  buttonBar = new QGridLayout(1,7,1);
	commandLine = new QGridLayout(1,2,1);	
  splitView = new QSplitter(this);
  splitView->setStyle(new QMotifStyle);
	QString dir1,dir2;
	config = kapp->config();
	config->setGroup("LastDirs");
	QDir dir_temp;
	dir2 = dir_temp.homeDirPath();
	dir1 = config->readEntry("LastDir1","/");
	dir2 = config->readEntry("LastDir2",dir2);
	viewer3 = new FileBasisView(true,dir2,"*",splitView,"view3");
  connect(viewer3,SIGNAL(changeStatus(const QString &)),SLOT(slotChangeStatus(const QString &)));
	
	viewer3->setViewFocus();
	viewer1 = new FileBasisView(true,dir1,"*",splitView,"view1");
  connect(viewer1,SIGNAL(changeStatus(const QString &)),SLOT(slotChangeStatus(const QString &)));
	viewer1->setViewFocus();
			  KLibFactory *m_factory;
			  m_factory = KLibLoader::self()->factory("libkonsolepart");
			  QObject *obj;
			  obj = static_cast<KParts::Factory *>(m_factory)->createPart(splitView);
			  konsolePart = static_cast<KParts::ReadOnlyPart *>(obj);
			  konsolePart->widget()->hide();
  mainLayout->addWidget(splitView,8);
  commandoBox = new KComboBox(true,this,"comBox");
//Kommando !!!!	
	commandoBox->setInsertionPolicy(QComboBox::AtTop);
	commandoBox->setMaximumSize(1200,20);
	commandoBox->setMinimumSize(400,20);
//	commandoBox->setFocusPolicy(StrongFocus);
	commandoBox->setFocusPolicy(NoFocus);
	commandoLabel = new QLabel("Label",this,"comLabel");
	commandoLabel->setAlignment(AlignRight|AlignBottom);
	commandoLabel->setMaximumSize(1200,20);
	commandoLabel->setMinimumSize(200,20);
  commandLine->addWidget(commandoLabel,0,0);
  commandLine->addWidget(commandoBox,0,1);
	mainLayout->addLayout(commandLine);
	
  initButtonBar();
  mainLayout->addLayout(buttonBar);
  mainLayout->activate();
	connect(parent,SIGNAL(keyPress(QKeyEvent *)),SLOT(keyPressed(QKeyEvent *)));
	connect(viewer1,SIGNAL(focussed()),SLOT(view1Focused()));
	connect(viewer3,SIGNAL(focussed()),SLOT(view2Focused()));
	connect(viewer1,SIGNAL(urlNew()),SLOT(changeComboLabel()));
	connect(viewer3,SIGNAL(urlNew()),SLOT(changeComboLabel()));
	connect(viewer1,SIGNAL(droppedFiles(QStringList)),SLOT(dropped(QStringList)));
	connect(viewer3,SIGNAL(droppedFiles(QStringList)),SLOT(dropped(QStringList)));
	connect(commandoBox,SIGNAL(returnPressed(const QString &)),this,SLOT(commandoInserted(const QString &)));
}

KCommander2View::~KCommander2View()
{
	config = kapp->config();
	config->setGroup("LastDirs");
	config->writeEntry("LastDir1", viewer1->currentDir());
	config->writeEntry("LastDir2", viewer3->currentDir());
}

void KCommander2View::view1Focused()
{
	temp = viewer1;
	noAktuell = viewer3;
	changeComboLabel();
	viewer1->changeFocusState(true);
	viewer3->changeFocusState(false);
	viewer1->focus->setPalette(white);
	viewer3->focus->setPalette(lightGray);
}

void KCommander2View::view2Focused()
{
	temp = viewer3;
	noAktuell = viewer1;
	changeComboLabel();
	viewer1->changeFocusState(false);
	viewer3->changeFocusState(true);
	viewer3->focus->setPalette(white);
	viewer1->focus->setPalette(lightGray);
}

KCommander2Doc *KCommander2View::getDocument() const
{
  KCommander2App *theApp=(KCommander2App *) parentWidget();

  return theApp->getDocument();
}

void KCommander2View::print(QPrinter *pPrinter)
{
  QPainter printpainter;
  printpainter.begin(pPrinter);
	
  // TODO: add your printing code here

//  printpainter.end();
}

void KCommander2View::initButtonBar()
{
  KCommander2App *theApp=(KCommander2App *) parentWidget();
	QString s;
		QFont f, def;
  	config = kapp->config();
	  config->setGroup("FontConfig");
	  f = config->readFontEntry("Font_1",&def);
	for(int i=0;i<7;i++)
	{
		s.sprintf("bt%d",i);
		buttonLeiste[i] = new QToolButton(this,s);
		buttonLeiste[i]->setMaximumSize(1100,25);
		buttonLeiste[i]->setFocusPolicy(NoFocus);
		// Font aus der config setzen
		buttonLeiste[i]->setFont(f);
		
		if(i<6)
		  connect(buttonLeiste[i], SIGNAL(clicked()), this, SLOT(buttonBarPressed()));
	}
	connect(buttonLeiste[6],SIGNAL(clicked()),theApp,SLOT(appHelpActivated()));
		
	
	QPopupMenu *pu = new QPopupMenu(this);
	int puID = 1;
	pu->insertItem("as Text",puID++);
	pu->insertItem("as HTML",puID++);
	pu->insertItem("as hexadecimal",puID++);
	
	buttonLeiste[0]->setText(i18n("F3 Show +"));
	buttonLeiste[0]->setPopup(pu);
	// connect(pu.......)
	
	pu = new QPopupMenu(this);
	puID = 1;
	pu->insertItem("with KEdit",puID++);
	pu->insertItem("with KWord",puID++);

	// for edit we want to insert configurabel editors
	buttonLeiste[1]->setText(i18n("F4 Edit +"));
	buttonLeiste[1]->setPopup(pu);
	
	
	buttonLeiste[2]->setText(i18n("F5 Copy"));
	buttonLeiste[3]->setText(i18n("F6 Move"));
	buttonLeiste[4]->setText(i18n("F7 MkDir"));
	buttonLeiste[5]->setText(i18n("F8 Delete"));
	buttonLeiste[6]->setText(i18n("F9 Help"));

	buttonBar->addWidget(buttonLeiste[0],0,0);
	buttonBar->addWidget(buttonLeiste[1],0,1);
	buttonBar->addWidget(buttonLeiste[2],0,2);
	buttonBar->addWidget(buttonLeiste[3],0,3);
	buttonBar->addWidget(buttonLeiste[4],0,4);
	buttonBar->addWidget(buttonLeiste[5],0,5);
	buttonBar->addWidget(buttonLeiste[6],0,6);
}

void KCommander2View::keyPressed(QKeyEvent *e)
{
	
	
//************************************************************************************	
//
//
//  Diese verkackte Routine muss neu geschrieben werden
//  Kein Schwein blickt hier mehr durch hehe
//
//
//************************************************************************************	
	
	
	
	int state = e->state();
//	bool	shift = false;
	bool SHIFT = false;
	bool CTRL = false;
	bool ALT = false;
	bool SHIFT_CTRL = false;
	bool ALT_CTRL = false;
	bool ALT_SHIFT_CTRL = false;
	bool ALT_SHIFT = false;
	
	// PREINSTALL of the variables
	if(state == 8)	  SHIFT = true;
	if(state == 16)	  CTRL = true;
	if(state == 32)   ALT = true;
	if(state == 24)   SHIFT_CTRL = true;	
  if(state == 40 )  ALT_SHIFT = true;
  if(state == 56 )  ALT_SHIFT_CTRL = true;
  if(state == 48 )  ALT_CTRL = true;
		
//	QString s;
	int key = e->key();
	
  // First we switch between ALT, CTRL & SHIFT
	
  if(CTRL)
  {
  	
  	//*************************************************
  	// Terminal Emulation on the inactive window
  	//*************************************************
  	if(key == 'O')  // CTRL+O = Terminal
  	{
			QValueList<int> splitValues;
			splitValues.append(300);
			splitValues.append(300);
	
  		//Hier das nicht aktuelle ListViewFenster ausblenden und Konsole rein
  		//dabei Flag setzen, so das beim naechsten Mal das Ganze rueckgaengig
  		//gemacht werden kann !!!!!!!!!!!!!!
		  // Als weiteres muss noch CTRL-D abgefangen werden. Das ist ein Signal vom System denke ich mal
		  if (konsoleFlag)
		  {
		  	if(konsolePart)
		  	  konsolePart->widget()->hide();
				if(!viewer1->widget()->isVisible())
					viewer1->widget()->show();
				else
					viewer3->widget()->show();
				splitView->setSizes(splitValues);
	  	konsoleFlag = false;
		  }
		  else
		  {
			  if(noAktuell == viewer1)
			  	viewer1->widget()->hide();
			  else
			  	viewer3->widget()->hide();
				konsolePart->openURL(temp->currentDir());
				konsolePart->widget()->show();
			  splitView->moveToFirst(konsolePart->widget());
				splitView->setSizes(splitValues);
				konsoleFlag = true;
			}
  	}
    if(key == '1')
    {
      QPopupMenu * popup = new QPopupMenu(0,"1");
      fillDriveAssigns(popup);
  	  connect(popup,SIGNAL(activated(int)),this,SLOT(drivePopupActivated ( int )));
  		popup->popup(QPoint(kapp->activeWindow()->x(),kapp->activeWindow()->y()));
    }

    if(key == '2')
    {
 	    QPopupMenu * popup = new QPopupMenu(0,"2");
      fillDriveAssigns(popup);
		  connect(popup,SIGNAL(activated(int)),this,SLOT(drivePopupActivated ( int )));
  		popup->popup(QPoint(kapp->activeWindow()->x(),kapp->activeWindow()->y()));
    }


    return; // if STRG was pressed, we return !!
  }
  //***************************************************************************************
  // END OF CTRL-KEYS
  //***************************************************************************************

  if(SHIFT)
  {
    return;
  }
  if(ALT)
  {
 	  QString s = temp->selectedFiles()->first()->text();
 	  commandoBox->setEditText(commandoBox->currentText() + s + " ");
    return;
  }

  if(SHIFT_CTRL)
  {
    if(key > 255)
    {
      key &= 0xff;
      if(key == 52)
      {
      }
    }
    return;
  //***************************************************************************************
  // END OF SHIFT+CTRL-KEYS
  //***************************************************************************************
  }
  if(ALT_CTRL)
  {
    return;
  //***************************************************************************************
  // END OF ALT+CTRL-KEYS
  //***************************************************************************************
  }
  if(ALT_SHIFT)
  {
    return;
  //***************************************************************************************
  // END OF ALT+SHIFT-KEYS
  //***************************************************************************************
  }
  if(ALT_SHIFT_CTRL)
  {
    return;
  //***************************************************************************************
  // END OF ALT+SHIFT+CTRL-KEYS
  //***************************************************************************************
  }


  //***************************************************************************************
  // here we come only if no extrakey was pressed
  //***************************************************************************************


 	if(key == 4100 /* ENTER */)
 	{
     KFileViewItem *test = temp->selectedFiles()->first();
     if(test) {
   	  if(!test->isFile())	{
     	  if(test->text() != "..")
       	  temp->setURL(KURL(temp->currentDir() + test->text()));
       	else
           temp->cdUp();
         return;
     	}
     	else {
     	  // it`s a file, so we execute it
     	  temp->fileSelected(test);
    	    return;
     	}
     }
     else {
       temp->cdUp();
  	    return;
     }
 	}



	if(key == '+')
			makeSelection();

  //***************************************************************************************
  // FUNCTION-KEYS
  //***************************************************************************************
	if(key > 255)
	{
		// Hier werden die Sondertasten behandelt
		key &= 0xff;
		
		switch (key){
   		case 50:{
    			if(!temp->selectedFiles()->isEmpty())typeFirstSelected(temp->selectedFiles());
    			}
    			break;
   		case 51:{
    			editFirstSelected(temp->selectedFiles());
    			}
    			break;
   		case 52:{
    			copyThis(temp->selectedFiles());
    			}
    			break;
   		case 53:{
    			moveThis(temp->selectedFiles());
    			}
    			break;
   		case 54:{
    			temp->mkDir();
    			}
    			break;
   		case 55:
    			deleteThis(temp->selectedFiles());
    			break;
		}
	}
   else
   {
		key = e->ascii();
		
		if(key == 32)		// Space gedrckt ??
		{
	  }
   }
}	

void KCommander2View::buttonBarPressed()
{
	QString key;
	QPushButton *s = (QPushButton *)sender();
	key = s->text();
	key = key.left(2);
	if(key=="F7")
	  temp->mkDir();
	if(key=="F8")
	  deleteThis(temp->selectedFiles());
	if(key=="F3")
	  typeFirstSelected(temp->selectedFiles());
  if(key=="F5")
    copyThis(temp->selectedFiles());
  if(key=="F6")
    moveThis(temp->selectedFiles());
  if(key=="F4")
    editFirstSelected(temp->selectedFiles());
//	LVkeyPressed(key);
}
  								
///// This functions to the main things  ///////////////////////////////////////////////////////////////////////////////7
void KCommander2View::makeDetailView()
{
	temp->setMode(KFile::Detail);
}

void KCommander2View::makeShortView()
{
	temp->setMode(KFile::Simple);
}

void KCommander2View::showHidden(bool show)
{
	temp->setToggleHidden(show);
}

void KCommander2View::deleteThis(KFileViewItemList *delList)
{
	if(delList->isEmpty()) return;
  config=kapp->config();
	config->setGroup("Delete");
	QString ask = config->readEntry("ask","yes");
	if(ask == "yes")
	{
		dlgDelete *askView = new dlgDelete();
		if(askView->exec() == 0){delete askView; return;};
		if (askView->QCheckBox_1->isChecked()) config->writeEntry("ask","no");
		else config->writeEntry("ask","yes");
	  delete askView;		
	}
  QListIterator<KFileViewItem> kit ( *delList );
  KURL::List tmp_dellist;
  kit.toFirst();
  for( ; kit.current(); ++kit )
  {
  	tmp_dellist.append((*kit)->url());
  }
   KIO::DeleteJob *job = new KIO::DeleteJob(tmp_dellist,false,true );
   connect( job, SIGNAL( result( KIO::Job* ) ),
            this, SLOT( slotResultDelete( KIO::Job* ) ) );
//   delete job;
}

void KCommander2View::copyThis(KFileViewItemList *copyList)
{
	if(copyList->isEmpty()) return;
	QString cpString,destString;
	copy = new dlgCopy();

  config=kapp->config();
	config->setGroup("Copy");
	QString ask = config->readEntry("ask","yes");
	
	destString = noAktuell->currentDir();

  // hier noch Quellpath setzen
  copy->setSourcePath(temp->currentDir());

  copy->setDestPath(destString);
	
	if(ask == "yes")
		if(copy->exec() == 0) // Abbruch !!
		  {
		    delete copy;
		    return;
		  };
		
  destString = copy->getDestPath();

  QListIterator<KFileViewItem> kit ( *copyList );
  KURL::List tmp_copylist;
  kit.toFirst();
  for( ; kit.current(); ++kit )
  {
  	tmp_copylist.append((*kit)->url());
  }
   const KURL destUrl(destString);
   KIO::Job * job = KIO::copy(tmp_copylist,destUrl,true );
   connect( job, SIGNAL( result( KIO::Job * ) ),
            this, SLOT( slotResultCopy( KIO::Job * ) ) );
	delete copy;
}

void KCommander2View::moveThis(KFileViewItemList *moveList)
{
	if(moveList->isEmpty()) return;
	QString cpString,destString;
//	copy = new dlgCopy();
//
//  config=kapp->config();
//	config->setGroup("Copy");
//	QString ask = config->readEntry("ask","yes");
//
//	if(ask == "yes")copy->exec();
	destString = noAktuell->currentDir();
  QListIterator<KFileViewItem> kit ( *moveList );
  KURL::List tmp_movelist;
  kit.toFirst();
  for( ; kit.current(); ++kit )
  {
  	tmp_movelist.append((*kit)->url());
  }
  const KURL destUrl(destString);

  KIO::CopyJob *job = new KIO::CopyJob(tmp_movelist,destUrl,KIO::CopyJob::Move,false,true );
  connect( job, SIGNAL( result( KIO::Job* ) ),
           this, SLOT( slotResultDelete( KIO::Job* ) ) );
//   delete job;
}

void KCommander2View::typeFirstSelected(KFileViewItemList *list)
{
	editor = new ownShow();
  editor->setGeometry(0,0,600,400);
  if(list->first() != NULL)
  {
	  QString tmp(list->first()->urlString());
		tmp = tmp.right(tmp.length()-1-tmp.find(":"));
 	  QFile f(tmp);
				f.open(IO_ReadOnly);
				QTextStream t( &f );
		editor->insertText(&t);
		editor->show();
	}
  return;
}

void KCommander2View::editFirstSelected(KFileViewItemList *editList)
{
	if(editList->isEmpty()) return;
	KShellProcess proc("bash");
	proc << "kwrite"<< editList->first()->urlString();
	proc.start(KProcess::DontCare,KProcess::NoCommunication);
}
	
void KCommander2View::setURL(const KURL &url)
{
	temp->setURL(url,true);
}
void KCommander2View::slotResultCopy( KIO::Job * )
{
}
void KCommander2View::slotResultDelete( KIO::Job* )
{
}
KFileViewItemList* KCommander2View::fileList()
{
	return temp->selectedFiles();
}

KFileItem* KCommander2View::findByName( const QString& name ) const
{
	return temp->findByName(name);
}

void KCommander2View::changeComboLabel()
{
	QString path(temp->currentDir());
	QString newPath;
	if(path.left(6) != "ftp://")
	{
		newPath = path;
	}
	else
	{
		// remove name & password from ftp-string
		newPath = path.right(path.length()-6);
		newPath = newPath.right(newPath.length()-newPath.find("@")-1);
		newPath = "ftp://" + newPath;
	}
	commandoLabel->setText(newPath + "    ");
}

void KCommander2View::dropped(QStringList list)
{
	if(sender()->name() == temp->name())
		return;
	KURL::List uList;
  for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
  {
		uList.append(KURL(*it));
	}
	QString destString = noAktuell->currentDir();
  const KURL destUrl(destString);
  KIO::Job * job = KIO::copy(uList,destUrl,true );
  connect( job, SIGNAL( result( KIO::Job * ) ),SLOT( slotResultCopy( KIO::Job * ) ) );
}
//************************************************************
// System Info View`s BEGIN
//************************************************************
void KCommander2View::showCPU()
{
  if(statusView)
    delete statusView;
  statusView = new PluginBasisView(1,this);
	statusView->setGeometry(temp->widget()->x(),temp->widget()->y(),temp->widget()->width(),temp->widget()->height());
	statusView->show();
	temp->setViewFocus();
}
void KCommander2View::showMEM()
{
  if(statusView)
    delete statusView;
  statusView = new PluginBasisView(2,this);
	statusView->setGeometry(temp->widget()->x(),temp->widget()->y(),temp->widget()->width(),temp->widget()->height());
	statusView->show();
	temp->setViewFocus();
}
void KCommander2View::showDRIVES()
{
  if(statusView)
    delete statusView;
  statusView = new PluginBasisView(3,this);
	statusView->setGeometry(temp->widget()->x(),temp->widget()->y(),temp->widget()->width(),temp->widget()->height());
	statusView->show();
	temp->setViewFocus();
}
void KCommander2View::resizeEvent(QResizeEvent *)
{
	if(statusView)
  	statusView->setGeometry(temp->widget()->x(),temp->widget()->y(),temp->widget()->width(),temp->widget()->height());
}
void KCommander2View::slotChangeStatus(const QString &statusText)
{
  emit changeStatus(statusText);
}

