/***************************************************************************
                          kcommander2view.h  -  description
                             -------------------
    begin                : Sun Sep 17 12:25:22 GMT 2000
    copyright            : (C) 2000 by Rene Mrten / Christian Fricke
    email                : cfricke@codewizards.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KCOMMANDER2VIEW_H
#define KCOMMANDER2VIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

#include "filebasisview.h"
#include "dlgdelete.h"
#include "dlgcopy.h"
#include "ownshow.h"
// include files for Qt
#include <qwidget.h>
//#include <qlabel.h>
#include <qsplitter.h>
#include <mysplitter.h>

#include <keditcl.h>
#include "options.h"
#include <klibloader.h>
#include <kparts/factory.h>
#include <kparts/part.h>

#include "pluginbasisview.h"

class KCommander2Doc;

class KCommander2View : public QWidget
{
  Q_OBJECT
  public:
    /** Constructor for the main view */
  KCommander2View(QWidget *parent = 0, const char *name=0);
    /** Destructor for the main view */
  ~KCommander2View();

    /** returns a pointer to the document connected to the view instance. Mind that this method requires a KCommander2App instance as a parent
     * widget to get to the window document pointer by calling the KCommander2App::getDocument() method.
     *
     * @see KCommander2App#getDocument
     */
  KCommander2Doc *getDocument() const;
	FileBasisView *temp,*noAktuell;
	
	PluginBasisView *pbv;
	
	void makeDetailView();
	void makeShortView();
	void showHidden(bool);
	void setURL(const KURL &url);	
    /** contains the implementation for printing functionality */
    void print(QPrinter *pPrinter);
 	KFileViewItemList* fileList();
	KFileItem* findByName( const QString& name ) const;
	static void fillDriveAssigns(QPopupMenu *menu);
  void swapViews();
	void makeSelection();
	void makeUnSelection();
	void makeOtherSelection(int);

	// System Info View
	void showCPU();
	void showMEM();
	void showDRIVES();
  PluginBasisView *statusView;
	
 private slots:
	void  keyPressed(QKeyEvent *);	
 	void view1Focused();
 	void view2Focused();
  void buttonBarPressed();
	void slotResultCopy( KIO::Job * ); 	 	
	void slotResultDelete( KIO::Job * ); 	 	
 	void changeComboLabel();
	void drivePopupActivated ( int );	
 	void dropped(QStringList);
	void commandoInserted(const QString &); 	
  void slotChangeStatus(const QString &);
	
	void resizeEvent(QResizeEvent *);
		
 signals:	
	void keyPress(QKeyEvent *);
  void changeStatus(const QString &);
	
  private:
	void initButtonBar();
	void deleteThis(KFileViewItemList *);
	void copyThis(KFileViewItemList *);
	void moveThis(KFileViewItemList *);
	void typeFirstSelected(KFileViewItemList *);
	void editFirstSelected(KFileViewItemList *);
	
  KConfig *config;
  QBoxLayout *mainLayout;
  QGridLayout *buttonBar,*commandLine;
  QToolButton	*buttonLeiste[7];
  QSplitter *splitView;
	FileBasisView *viewer1,*viewer3;
	KComboBox *commandoBox;
	QLabel *commandoLabel;
//	KEdit *editor;	
	ownShow *editor;
	dlgCopy *copy;
	bool konsoleFlag;
	KParts::ReadOnlyPart *konsolePart;
};

#endif // KCOMMANDER2VIEW_H
