/***************************************************************************
*                        kfilednddetailview.cpp  -  description
*                           -------------------
*  begin                : Wed Nov 1 2000
*  copyright            : (C) 2000 by Bjrn Sahlstrm
*  email                : bjorn@kbear.org
***************************************************************************/

/***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************/

#include "kfilednddetailview.h"

//-----------------------------------------------
KFileDndDetailView::KFileDndDetailView(QWidget *parent, const char *name )
	:	KFileDetailView(parent,name), dropItem(0),
		myAutoOpenTimer(this), autoOpenTime( 750 ),
		b_useAutoOpenTimer( true )
{
	setDragEnabled( true );
	setDropVisualizer( true );
	setAcceptDrops( true );
	viewport()->setAcceptDrops( true );
	connect( &myAutoOpenTimer, SIGNAL( timeout() ),this, SLOT( slotOpenFolder() ) );
}
//-----------------------------------------------
KFileDndDetailView::~KFileDndDetailView(){
}
//-----------------------------------------------
void KFileDndDetailView::slotOpenFolder(){
	myAutoOpenTimer.stop();
	if( !dropItem )
		return;
	const KFileViewItemList* fileList = KFileView::items();
	KFileViewItemListIterator it( *(QList<KFileViewItem>*)fileList );
	for( ; it.current() ;++it ){
		if( (*it)->name() == dropItem->text(0) && (*it)->isFile())
			return;
	}
	emit openFolder( dropItem->text(0) );
}
//-----------------------------------------------
void KFileDndDetailView::setAutoOpenTime( const int& time ){
	autoOpenTime = time;
	useAutoOpenTimer();
}
//-----------------------------------------------
void KFileDndDetailView::useAutoOpenTimer( bool use ){
	b_useAutoOpenTimer = use;
	if( ! use ) {
		dropItem = 0;
		myAutoOpenTimer.stop();
	}
}
//-----------------------------------------------
void KFileDndDetailView::contentsDragEnterEvent( QDragEnterEvent *e ) {
	setSelectionMode( KFile::Single );
	if ( QUriDrag::canDecode( e ) ) {
		e->accept();
		QListViewItem *i = itemAt( contentsToViewport( e->pos() ) );
		if ( i && b_useAutoOpenTimer ) {
			dropItem = i;
			myAutoOpenTimer.start( autoOpenTime );
		}
	}
}
//-----------------------------------------------
void KFileDndDetailView::contentsDragMoveEvent( QDragMoveEvent *e ) {
	if ( QUriDrag::canDecode( e ) ) {
		e->accept();
		QListViewItem *i = itemAt( contentsToViewport(  e->pos() ) );
		if ( i ) {
			if ( i != dropItem && b_useAutoOpenTimer ) {
				myAutoOpenTimer.stop();
				dropItem = i;
				myAutoOpenTimer.start( autoOpenTime );
			}
		}
		else
			myAutoOpenTimer.stop();
	}
}
//-----------------------------------------------
void KFileDndDetailView::contentsDragLeaveEvent( QDragLeaveEvent* ) {
	if( b_useAutoOpenTimer ) {
		myAutoOpenTimer.stop();
		dropItem = 0;
	}
	setSelectionMode( KFile::Extended );
}
//-----------------------------------------------
void KFileDndDetailView::contentsDropEvent( QDropEvent* e ) {
	setSelectionMode( KFile::Extended );
	if( b_useAutoOpenTimer ) {
		myAutoOpenTimer.stop();
		dropItem = 0;
	}
	if ( QUriDrag:: canDecode( e ) ) {
		e->accept();
		QListViewItem *afterme;
		QPoint p (contentsToViewport( e->pos() ));

		// Get the position to put it in
		QListViewItem *atpos = itemAt( p );

		if ( ! atpos ) // if no item
			afterme = lastItem();
		else { // get the closest one
			// That is, the space between two listviewitems
			// Since this aims to be user-friendly :)
			int y = mapFromGlobal(p).y() - mapFromGlobal(itemRect(atpos).topLeft()).y();
			afterme = y < (atpos->height()/2) ? atpos->itemAbove() : atpos;
		}
		QStringList urlList;    // to the outside world
		QUriDrag::decodeToUnicodeUris( e, urlList );
		emit dropped( urlList );
	}
}
//-----------------------------------------------
void KFileDndDetailView::startDrag(){
	// create a list of the URL:s that we want to drag
	QStringList lst;
	const KFileViewItemList* fileList = KFileView::selectedItems();
	KFileViewItemListIterator it( *(QList<KFileViewItem>*)fileList );
	for ( ; it.current(); ++it ){
		lst << (*it)->url().prettyURL();
	}
	myDragObject = new QUriDrag( this );
	((QUriDrag*)myDragObject)->setUnicodeUris( lst );
	myDragObject->setPixmap( *(currentItem()->pixmap(0)) );
	myDragObject->dragCopy();   // start the drag
}
//-----------------------------------------------
