/***************************************************************************
                          kfiledndiconview.cpp  -  description
                             -------------------
    begin                : Sun Feb 4 2001
    copyright            : (C) 2001 by Rene Mrten & Christian Fricke
    email                : delta_x@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
*                        kfiledndiconview.cpp  -  description
*                           -------------------
*  begin                : Wed Nov 1 2000
*  copyright            : (C) 2000 by Bjrn Sahlstrm
*  email                : bjorn@kbear.org
***************************************************************************/

/***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************/

#include "kfiledndiconview.h"

//-----------------------------------------------
KFileDnDIconView::KFileDnDIconView( QWidget *parent, const char *name )
	: KFileIconView(parent,name), dropItem(0),
		myAutoOpenTimer(this), autoOpenTime( 750 ),
		b_useAutoOpenTimer( true )
{
	setAcceptDrops( true );
	viewport()->setAcceptDrops( true );
	connect( &myAutoOpenTimer, SIGNAL( timeout() ), this, SLOT( slotOpenFolder() ) );
}
//-----------------------------------------------
KFileDnDIconView::~KFileDnDIconView(){
}
//-----------------------------------------------
void KFileDnDIconView::slotOpenFolder(){
	myAutoOpenTimer.stop();
	if( !dropItem )
		return;
	const KFileViewItemList* fileList = KFileView::items();
	KFileViewItemListIterator it( *(QList<KFileViewItem>*)fileList );
	for( ; it.current() ;++it ){
		if( (*it)->name() == dropItem->text() && (*it)->isFile())
			return;
	}
	emit openFolder( dropItem->text() );
}
//-----------------------------------------------
void KFileDnDIconView::setAutoOpenTime( const int& time ){
	autoOpenTime = time;
	useAutoOpenTimer();
}
//-----------------------------------------------
void KFileDnDIconView::useAutoOpenTimer( bool use ){
	b_useAutoOpenTimer = use;
}
//-----------------------------------------------
void KFileDnDIconView::contentsDragEnterEvent( QDragEnterEvent *e ) {
	setSelectionMode( KFile::Single );  // change selection mode to single, otherwhise we get a very strange behaviot ;)
	if ( QUriDrag::canDecode( e ) ) { // can we decode this ?
		e->accept( true );            // Yes
		QIconViewItem *i = findItem( contentsToViewport( e->pos() ) );
		if ( i ) {  // are we over an item ?
			dropItem = i;  // set new dropItem
			myAutoOpenTimer.start( autoOpenTime ); // restart timer
		}
	}
}
//-----------------------------------------------
void KFileDnDIconView::contentsDragMoveEvent( QDragMoveEvent *e ) {
	if ( QUriDrag::canDecode( e ) ) { // can we decode this ?
		e->accept( true );     // Yes
		QIconViewItem *i = findItem( contentsToViewport( e->pos() ) );
		if ( i ) {  // are we over an item ?
			if ( i != dropItem ) { // if so, is it a new one ?
				myAutoOpenTimer.stop(); // stop timer
				dropItem = i;      // set new dropItem
				myAutoOpenTimer.start( autoOpenTime ); // restart timer
			}
		}
		else
			myAutoOpenTimer.stop(); // stop timer
	}
}
//-----------------------------------------------
void KFileDnDIconView::contentsDragLeaveEvent( QDragLeaveEvent* ) {
	myAutoOpenTimer.stop(); // stop timer
	dropItem = 0;                  // clear dropItem
	setSelectionMode( KFile::Extended ); // change selection mode back to extended
}
//-----------------------------------------------
void KFileDnDIconView::contentsDropEvent( QDropEvent* e ) {
	setSelectionMode( KFile::Extended );
	myAutoOpenTimer.stop();
	dropItem = 0;
	KFileIconView::contentsDropEvent( e );
	if( QUriDrag::canDecode( e ) ) { // the drop was accepted so lets emit this
		QStringList urlList;    // to the outside world
		QUriDrag::decodeToUnicodeUris( e, urlList );
		emit dropped( urlList );
	}
}
//-----------------------------------------------
void KFileDnDIconView::startDrag(){
	if ( ! currentItem() ) // is there any selected items ?
		return;              // nope
	dragObject();        // create the drag object and set it's pixmap
	myDragObject->setPixmap( *currentItem()->pixmap(),
 		     								QPoint( currentItem()->pixmapRect().width() / 2,
 		     								currentItem()->pixmapRect().height() / 2 ) );
	myDragObject->dragCopy();   // start the drag
}
//-----------------------------------------------
QDragObject* KFileDnDIconView::dragObject() {
	myDragObject = new KFileIconDrag( viewport(), "MyIconDrag" );
	QPoint orig = viewportToContents( viewport()->mapFromGlobal( QCursor::pos() ) );
	QStringList lst;
	// get the selected items to fill the drag object with
	const KFileViewItemList* fileList = KFileView::selectedItems();
	KFileViewItemListIterator it( *(QList<KFileViewItem>*)fileList );
	
	for ( ; it.current(); ++it ){
		QIconDragItem id;
		id.setData( QCString( (*it)->name() ) );
		KFileIconViewItem* i = (KFileIconViewItem*)(*it);
		((KFileIconDrag*)myDragObject)->append( id,
				QRect( i->pixmapRect( false ).x() - orig.x(),
				i->pixmapRect( false ).y() - orig.y(),
				i->pixmapRect().width(), i->pixmapRect().height() ),
				QRect( i->textRect( false ).x() - orig.x(),
				i->textRect( false ).y() - orig.y(),
				i->textRect().width(), i->textRect().height() ),
				QString( (*it)->url().prettyURL() ));
	}
	return myDragObject;
}
//-----------------------------------------------
