/***************************************************************************
                          pluginbasisview.cpp  -  description
                             -------------------
    begin                : Sat Apr 14 2001
    copyright            : (C) 2001 by Christian Fricke & Rene Maerten
    email                : cfricke@kcommander.org
 ***************************************************************************/
// $Revision: 1.11 $
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "pluginbasisview.h"
#include <qlistview.h>
#include <qfile.h>
#include <kapp.h>
#include <kprocess.h>

// our own ListItem
extListViewItem::extListViewItem(QListView *parent,const char *s1,const char *s2,const char *s3, const char *s4,const char *s5,const char *s6,const char *s7,const char *s8)
                    :QListViewItem(parent,s1,s2,s3,s4,s5,s6,s7,s8)
{
  pegel = 0;
}

void extListViewItem::paintCell (QPainter * p,const QColorGroup &cg,int column, int width,int align)
{
    if(column != 2)
      QListViewItem::paintCell (p,cg,column,width,align);
    else
    {
       p->setPen(cg.dark());
       p->drawRect(0,0,width,height());
       p->fillRect(1,1,width-2,height()-2,cg.base());
       int b = width-2;
       b = pegel * b / 100;
       p->fillRect(1,1,b,height()-2,/*cg.midlight()*/blue);
       QString s; s.sprintf("%d %%",pegel);
       p->setPen(/*cg.text()*/white);
       p->drawText(0,0,width-1,height()-1,AlignCenter,s);
       p->setPen(/*cg.text()*/black);
       p->drawText(1,1,width-1,height()-1,AlignCenter,s);
    }
}
void extListViewItem::paintFocus(QPainter *p, const QColorGroup &cg, const QRect &r)
{
  QListViewItem::paintFocus(p, cg, r);
}


PluginBasisView::PluginBasisView(int index,QWidget *parent): KListView(parent)
{
  setSorting(-1);
  appIndex = index;
  switch(index)
  {
    case 1: // CPU Info
            setColumns("Description\tResult");
            addItem("Bogomips","");
            addItem("CPU Flags","");
            addItem("WP","");
            addItem("CPU ID Level","");
            addItem("FPU Exception","");
            addItem("FPU","");
            addItem("Coma Bug","");
            addItem("0xF00F Bug","");
            addItem("Sep Bug","");
            addItem("Hlt Bug","");
            addItem("FDiv Bug","");
            addItem("Cache Size","");
            addItem("CPU MHz","");
            addItem("Stepping","");
            addItem("Model Name","");
            addItem("Model ID","");
            addItem("CPU Family","");
            addItem("Vendor ID","");
            addItem("Processors","");
            break;

    case 2: // Memoryinfo
            setColumns("Description\tSize\tPercent");
            addItem("Swap free","0kB");
            addItem("Swap total","0kB");
            addItem("Big memory free","0kB");
            addItem("Big memory total    ","0kB");
            addItem("Cached","0kB");
            addItem("Buffers","0kB");
            addItem("Shared memory","0kB");
            addItem("Free memory","0kB");
            addItem("Total memory  ","0kB");

            break;
    case 3: // Drive Infos
            setColumns("Drive\tDevice\tFull Percentage");
            // Read mounted Devices from "df" command
            KShellProcess proc("bash");
            proc << "df" << ">" << "/tmp/.kcdrives";
          	proc.start(KProcess::DontCare,KProcess::NoCommunication);
            QFile file("/tmp/.kcdrives");
            bool id = file.open(IO_ReadOnly);
            // now we count the number of drives mounted
            drives = 0;
            QString line;
            QString str_drives;
            QString str_devices,t;
            if(id)
            {
              file.readLine(line,255);
              while(!file.atEnd())
              {
                file.readLine(line,255);
                t = line;
                t = t.left(20); // device
                t = t.simplifyWhiteSpace();
                str_devices =  t + "\t" + str_devices;
                line = line.mid(56,200);
                line = line.left(line.length()-1) + "\t";
                str_drives = line + str_drives;
              }

              QString s(str_drives), rest;
              QString s2(str_devices), rest2;
              do
              {
                if(s.find("\t") == -1)
                {
                  addItem(s,s2);
                  s = ""; s2 = "";
                }
                else
                {
                  rest = s.right(s.length()-s.find("\t")-1);
                  rest2 = s2.right(s2.length()-s2.find("\t")-1);
                  s = s.left(s.find("\t"));
                  s2 = s2.left(s2.find("\t"));
                  addItem(s,s2);
                  s = rest;
                  s2 = rest2;
                }
              } while(s.length() != 0);
            }

            break;
  }
  startTimer(1500);
}
PluginBasisView::~PluginBasisView(){
}

void PluginBasisView::setColumns(const char *text)
{
    // Columntext separated with tabs
    QString s(text), rest;
    do
    {
      if(s.find("\t") == -1)
      {
        addColumn(s);
        s = "";
      }
      else
      {
        rest = s.right(s.length()-s.find("\t")-1);
        s = s.left(s.find("\t"));
        addColumn(s);
        s = rest;
      }
    } while(s.length() != 0);
}

void PluginBasisView::addItem(const char *s1, const char *s2)
{
    extListViewItem *item = new extListViewItem((QListView *)this,s1,s2);
    item = 0L;
}
void PluginBasisView::timerEvent(QTimerEvent *te)
{
  te = 0L;
  switch(appIndex)
  {
    case 1:
            updateCPUinfo();
            break;
    case 2:
            updateMemory();
            break;
    case 3:
            updateDrives();
            break;
  }
}

void PluginBasisView::updateCPUinfo()
{
  QListViewItemIterator it(this);
  QFile file("/proc/cpuinfo");
  QString s;
  extListViewItem *fi = (extListViewItem *) it.current();
//  int temp;
  if(file.open(IO_ReadOnly))
  {
    for(int i=0;i<19;i++)
    {
      kapp->processEvents();
      file.readLine(s,255);
      s = s.right(s.length()-s.find(":")-1);
      s = s.left(s.length()-1);
      if(i == 0)
      {
         // Count of processors
         int j = s.toInt()+1;
         s.sprintf("%d",j);
      }
      fi->setText(1,s);
      it++;
      fi = (extListViewItem *)it.current();
    }
    file.close();
  }

}
void PluginBasisView::updateMemory()
{
  QListViewItemIterator it(this);
  QFile file("/proc/meminfo");
  QString s;
  extListViewItem *fi = (extListViewItem *) it.current();
  int memTotal = 0;
  int bigTotal = 0;
  int swapTotal = 0;
  int temp;
  if(file.open(IO_ReadOnly))
  {
    file.readLine(s,255);
    file.readLine(s,255);
    file.readLine(s,255);

    for(int i=0;i<9;i++)
    {
      kapp->processEvents();
      file.readLine(s,255);
      s = s.simplifyWhiteSpace();
      s = s.right(s.length()-s.find(" ")-1);
      s = s.left(s.find(" "));
      switch(i)
      {
        case 0:
                memTotal = s.toInt();
                fi->setPercent(100);
                break;
        case 1:
                temp = s.toInt(); // speicherverbrauch
                temp = temp * 100 / memTotal;
                fi->setPercent(temp);
                break;
        case 2:
                temp = s.toInt(); // speicherverbrauch
                temp = temp * 100 / memTotal;
                fi->setPercent(temp);
                break;
        case 3:
                temp = s.toInt(); // speicherverbrauch
                temp = temp * 100 / memTotal;
                fi->setPercent(temp);
                break;
        case 4:
                temp = s.toInt(); // speicherverbrauch
                temp = temp * 100 / memTotal;
                fi->setPercent(temp);
                break;
        case 5: bigTotal = s.toInt();
                fi->setPercent(100);
                break;
        case 6:
                temp = s.toInt(); // speicherverbrauch
                temp = (int)(temp * 100 / (bigTotal+.1));
                fi->setPercent(temp);
                break;
        case 7: swapTotal = s.toInt();
                fi->setPercent(100);
                break;
        case 8:
                temp = s.toInt(); // speicherverbrauch
                temp = (int)(temp * 100 / (swapTotal+.1));
                fi->setPercent(temp);
                break;
      }
      fi->setText(1,"  " + s + "  ");
      it++;
      fi = (extListViewItem *)it.current();
    }
    file.close();
  }

}

void PluginBasisView::updateDrives()
{
  KShellProcess proc("bash");
  proc << "df" << ">" << "/tmp/.kcdrives";
 	proc.start(KProcess::DontCare,KProcess::NoRead);
  QListViewItemIterator it(this);
  extListViewItem *fi = (extListViewItem *) it.current();
  QFile file("/tmp/.kcdrives");
  bool id=false;
  while(!id == file.open(IO_ReadOnly))
  {
  }
  QString s;
  if(id)
  {
    file.readLine(s,255);
    while(!file.atEnd())
    {
      file.readLine(s,255);
      debug(s);
      int perc = s.mid(51,3).toInt();
      fi->setPercent(perc);
      fi->repaint();
      it++;
      fi = (extListViewItem *)it.current();
    }
  }
}

